<?php

/*
 * Copyright (C) 2022 Intergo Telecom Ltd <support@sms.to>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

namespace Smsto;

defined('_JEXEC') or die;

use Exception;
use Joomla\CMS\Factory;
use Joomla\Http\Http;
use Joomla\Uri\Uri;

class Helper
{
        /**
         * Get user id from username
         * 
         * @author Panayiotis Halouvas <phalouvas@kainotomo.com>
         *
         * @param string $username
         * @return int
         * @throws Exception
         */
        public static function getUserId(string $username)
        {
                $db = Factory::getContainer()->get(\Joomla\Database\DatabaseInterface::class);

                $query = $db->getQuery(true);
                $query->select($db->quoteName(array('id')));
                $query->from($db->quoteName('#__users'));
                $query->where($db->quoteName('username') . ' LIKE ' . $db->quote($username));
                $db->setQuery($query);
                $user_id = $db->loadResult();
                if (!$user_id) {
                        throw new Exception('User not found ', 404);
                }

                $user = Factory::getUser($user_id);
                return $user->id;
        }

        /**
         * Get user 2fa phone
         * 
         * @author Panayiotis Halouvas <phalouvas@kainotomo.com>
         *
         * @param integer $user_id
         * @return string
         */
        public static function getUserPhone(int $user_id)
        {
                $user = Factory::getUser($user_id);
                if (!$user->otpKey) {
                        throw new Exception("User does not have any associated phone.", 417);
                }
                list($method, $config) = explode(':', $user->otpKey, 2);
                if ($method !== 'smsto') {
                        throw new Exception("User does not have any associated phone.", 417);
                }
                $config = json_decode($config);
                return $config->phonenumber;
        }

        /**
         * Send API call to smsto
         * 
         * @author Panayiotis Halouvas <phalouvas@kainotomo.com>
         *
         * @param string $api_key
         * @param string $to
         * @param string $message
         * @param string|null $sender_id
         * @param string|null $callback_url
         * @param boolean $bypass_optout
         * @return void
         */
        public static function send(string $api_key, string $to, string $message, string $sender_id = null, string $callback_url = null, bool $bypass_optout = false)
        {
                header('Content-Type: application/json'); // Specify the type of data
                $ch = curl_init('https://api.sms.to/sms/send'); // Initialise cURL

                $post = [];
                $post['to'] = $to;
                $post['message'] = $message;
                if ($bypass_optout) {
                        $post['bypass_optout'] = $bypass_optout;
                }
                if ($sender_id) {
                        $post['sender_id'] = $sender_id;
                }
                if ($callback_url) {
                        $post['callback_url'] = $callback_url;
                }

                $authorization = "Authorization: Bearer " . $api_key; // Prepare the authorisation token
                curl_setopt($ch, CURLOPT_HTTPHEADER, array('Content-Type: application/json', $authorization)); // Inject the token into the header
                curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
                curl_setopt($ch, CURLOPT_POST, 1); // Specify the request method as POST
                curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($post)); // Set the posted fields
                curl_setopt($ch, CURLOPT_FOLLOWLOCATION, 1); // This will follow any redirects
                $response = curl_exec($ch); // Execute the cURL statement
                curl_close($ch); // Close the cURL connection
        }

        /**
         * Method to call SMSto api
         * 
         * @author Panayiotis Halouvas <phalouvas@kainotomo.com>
         *
         * @param string $api_key
         * @param string $method
         * @param string $url
         * @param string|array|null $payload
         * @return void
         */
        public static function callSmsto(string $api_key, string $method, string $url, $payload = null)
        {
                $headers = [
                        'Authorization' => 'Bearer ' . $api_key,
                        'Content-Type' => 'application/json',
                ];
                $http = new Http();
                if ($method == 'get') {
                        if ($payload) {
                                $uri = new Uri($url);
                                $page = $uri->getVar('page');
                                $uri->setQuery(json_decode($payload, true));
                                $uri->setVar('page', $page);
                                $url = $uri->toString();
                                $url = str_replace('smsto_limit', 'limit', $url);
                        }
                        $response = $http->get($url, $headers);
                } 
                if ($method === 'put') {
                        $response = $http->put($url, $payload, $headers);
                }
                if ($method === 'patch') {
                        $response = $http->patch($url, $payload, $headers);
                }
                if ($method === 'post') {
                        $response = $http->post($url, $payload, $headers);
                }
                if ($method === 'delete') {
                        $response = $http->delete($url, $headers, null, $payload);
                }

                echo $response->body;
        }
}
