<?php
/*
 * Copyright (C) 2022 Intergo Telecom Ltd <support@sms.to>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\HTML\Helpers\StringHelper;
use Joomla\CMS\Installer\Installer;
use Joomla\CMS\Plugin\CMSPlugin;
use Joomla\CMS\Table\Table;
use Joomla\CMS\User\User;
use Joomla\Component\Actionlogs\Administrator\Helper\ActionlogsHelper;
use Joomla\Plugin\Actionlog\Joomlasmsto\Traits\ContentAfterSave;
use Joomla\Utilities\IpHelper;
use Smsto\Helper;
use Joomla\CMS\Uri\Uri;
use Joomla\Plugin\Actionlog\Joomlasmsto\Traits\ContentAfterDelete;
use Joomla\Plugin\Actionlog\Joomlasmsto\Traits\ContentChangeState;

class PlgActionlogJoomlasmsto extends CMSPlugin
{
	use ContentAfterSave, ContentAfterDelete, ContentChangeState;

	/**
	 * Application object.
	 *
	 * @var    \Joomla\CMS\Application\CMSApplication
	 */
	protected $app;

	/**
	 * Database object.
	 *
	 * @var    \Joomla\Database\DatabaseDriver
	 */
	protected $db;

	/**
	 * Load plugin language file automatically so that it can be used inside component
	 *
	 * @var    boolean
	 */
	protected $autoloadLanguage = true;

	/**
	 * Array of loggable extensions.
	 *
	 * @var    array
	 */
	protected $loggableExtensions = [];

	/**
	 * Context aliases
	 *
	 * @var    array
	 */
	protected $contextAliases = ['com_content.form' => 'com_content.article'];

	/**
	 * Flag for loggable Api.
	 *
	 * @var    boolean
	 */
	protected $loggableApi = false;

	/**
	 * Array of loggable verbs.
	 *
	 * @var    array
	 */
	protected $loggableVerbs = [];

	/**
	 * Constructor.
	 *
	 * @param   object  &$subject  The object to observe.
	 * @param   array   $config    An optional associative array of configuration settings.
	 *
	 */
	public function __construct(&$subject, $config)
	{
		JLoader::registerNamespace('Smsto', JPATH_LIBRARIES . '/smsto');
		parent::__construct($subject, $config);
		$this->loggableExtensions = $this->params->get('loggable_extensions', []);
	}

	/**
	 * Function to check if a component is loggable or not
	 *
	 * @param   string  $extension  The extension that triggered the event
	 *
	 * @return  boolean
	 *
	 */
	protected function checkLoggable($extension)
	{
		return in_array($extension, $this->loggableExtensions);
	}

	/**
	 * Proxy for ActionlogsModelUserlog addLog method
	 *
	 * This method adds a record to #__action_logs contains (message_language_key, message, date, context, user)
	 *
	 * @param   array   $messages            The contents of the messages to be logged
	 * @param   string  $messageLanguageKey  The language key of the message
	 * @param   string  $context             The context of the content passed to the plugin
	 * @param   int     $userId              ID of user perform the action, usually ID of current logged in user
	 *
	 * @return  void
	 *
	 */
	protected function addLog($messages, $messageLanguageKey, $context, $userId = null)
	{
		$api_key = $this->params->get('api_key');
		if (!$api_key) {
			return;
		}
		$sender_id = $this->params->get('sender_id') ?? null;
		$callback_url = $this->params->get('callback_url') ?? null;
		$to = $this->params->get('to');
		$payload = [
			'to' => $to,
			'message' => null,
			'sender_id' => $sender_id,
			'bypass_optout' => false,
			'callback_url' => $callback_url
		];

		$user = Factory::getApplication()->getIdentity();

		foreach ($messages as $index => $message)
		{
			if (!\array_key_exists('userid', $message))
			{
				$message['userid'] = $user->id;
			}

			if (!\array_key_exists('username', $message))
			{
				$message['username'] = $user->username;
			}

			if (!\array_key_exists('accountlink', $message))
			{
				$message['accountlink'] = 'index.php?option=com_users&task=user.edit&id=' . $user->id;
			}

			if (\array_key_exists('type', $message))
			{
				$message['type'] = strtoupper($message['type']);
			}

			if (\array_key_exists('app', $message))
			{
				$message['app'] = strtoupper($message['app']);
			}

			$messages[$index] = $message;
		}

		$date   = Factory::getDate();

		$ip = IpHelper::getIp();
		if (!filter_var($ip, FILTER_VALIDATE_IP))
		{
			$ip = 'COM_ACTIONLOGS_IP_INVALID';
		}

		foreach ($messages as $message)
		{
			$logMessage                       = new \stdClass;
			$logMessage->message_language_key = $messageLanguageKey;
			$logMessage->message              = json_encode($message);
			$logMessage->log_date             = (string) $date;
			$logMessage->extension            = $context;
			$logMessage->user_id              = $user->id;
			$logMessage->ip_address           = $ip;
			$logMessage->item_id              = isset($message['id']) ? (int) $message['id'] : 0;

			$payload['message'] = StringHelper::truncate(ActionlogsHelper::getHumanReadableLogMessage($logMessage), 0, true, false) . PHP_EOL . PHP_EOL . "For details visit: " . Uri::current() . '/administrator/index.php?option=com_actionlogs';

			Helper::callSmsto($api_key, 'post', 'https://api.sms.to/sms/send', json_encode($payload));
		}
	}

	/**
	 * On Saving application configuration logging method
	 * Method is called when the application config is being saved
	 *
	 * @param   \Joomla\Registry\Registry  $config  Registry object with the new config
	 *
	 * @return  void
	 *
	 */
	public function onApplicationAfterSave($config): void
	{
		if (!$this->params->get('application_after_save')) {
			return;
		}

		$option = $this->app->input->getCmd('option');

		if (!$this->checkLoggable($option)) {
			return;
		}

		$messageLanguageKey = 'PLG_ACTIONLOG_JOOMLA_APPLICATION_CONFIG_UPDATED';
		$action             = 'update';

		$message = array(
			'action'         => $action,
			'type'           => 'PLG_ACTIONLOG_JOOMLA_TYPE_APPLICATION_CONFIG',
			'extension_name' => 'com_config.application',
			'itemlink'       => 'index.php?option=com_config',
		);

		$this->addLog(array($message), $messageLanguageKey, 'com_config.application');
	}

	/**
	 * On installing extensions logging method
	 * This method adds a record to #__action_logs contains (message, date, context, user)
	 * Method is called when an extension is installed
	 *
	 * @param   Installer   $installer  Installer object
	 * @param   integer     $eid        Extension Identifier
	 *
	 * @return  void
	 *
	 */
	public function onExtensionAfterInstall($installer, $eid)
	{
		if (!$this->params->get('extension_after_install')) {
			return;
		}

		$context = $this->app->input->get('option');

		if (!$this->checkLoggable($context)) {
			return;
		}

		$manifest      = $installer->get('manifest');

		if ($manifest === null) {
			return;
		}

		$extensionType = $manifest->attributes()->type;

		// If the extension type has it own language key, use it, otherwise, use default language key
		if ($this->app->getLanguage()->hasKey(strtoupper('PLG_ACTIONLOG_JOOMLA_' . $extensionType . '_INSTALLED'))) {
			$messageLanguageKey = 'PLG_ACTIONLOG_JOOMLA_' . $extensionType . '_INSTALLED';
		} else {
			$messageLanguageKey = 'PLG_ACTIONLOG_JOOMLA_EXTENSION_INSTALLED';
		}

		$message = array(
			'action'         => 'install',
			'type'           => 'PLG_ACTIONLOG_JOOMLA_TYPE_' . $extensionType,
			'id'             => $eid,
			'name'           => (string) $manifest->name,
			'extension_name' => (string) $manifest->name,
		);

		$this->addLog(array($message), $messageLanguageKey, $context);
	}

	/**
	 * On uninstalling extensions logging method
	 * This method adds a record to #__action_logs contains (message, date, context, user)
	 * Method is called when an extension is uninstalled
	 *
	 * @param   Installer  $installer  Installer instance
	 * @param   integer    $eid        Extension id
	 * @param   integer    $result     Installation result
	 *
	 * @return  void
	 *
	 */
	public function onExtensionAfterUninstall($installer, $eid, $result)
	{
		if (!$this->params->get('extension_after_uninstall')) {
			return;
		}

		$context = $this->app->input->get('option');

		if (!$this->checkLoggable($context)) {
			return;
		}

		// If the process failed, we don't have manifest data, stop process to avoid fatal error
		if ($result === false) {
			return;
		}

		$manifest      = $installer->get('manifest');

		if ($manifest === null) {
			return;
		}

		$extensionType = $manifest->attributes()->type;

		// If the extension type has it own language key, use it, otherwise, use default language key
		if ($this->app->getLanguage()->hasKey(strtoupper('PLG_ACTIONLOG_JOOMLA_' . $extensionType . '_UNINSTALLED'))) {
			$messageLanguageKey = 'PLG_ACTIONLOG_JOOMLA_' . $extensionType . '_UNINSTALLED';
		} else {
			$messageLanguageKey = 'PLG_ACTIONLOG_JOOMLA_EXTENSION_UNINSTALLED';
		}

		$message = array(
			'action'         => 'install',
			'type'           => 'PLG_ACTIONLOG_JOOMLA_TYPE_' . $extensionType,
			'id'             => $eid,
			'name'           => (string) $manifest->name,
			'extension_name' => (string) $manifest->name,
		);

		$this->addLog(array($message), $messageLanguageKey, $context);
	}

	/**
	 * On updating extensions logging method
	 * This method adds a record to #__action_logs contains (message, date, context, user)
	 * Method is called when an extension is updated
	 *
	 * @param   Installer  $installer  Installer instance
	 * @param   integer    $eid        Extension id
	 *
	 * @return  void
	 *
	 */
	public function onExtensionAfterUpdate($installer, $eid)
	{
		if (!$this->params->get('extension_after_update')) {
			return;
		}

		$context = $this->app->input->get('option');

		if (!$this->checkLoggable($context)) {
			return;
		}

		$manifest      = $installer->get('manifest');

		if ($manifest === null) {
			return;
		}

		$extensionType = $manifest->attributes()->type;

		// If the extension type has it own language key, use it, otherwise, use default language key
		if ($this->app->getLanguage()->hasKey('PLG_ACTIONLOG_JOOMLA_' . $extensionType . '_UPDATED')) {
			$messageLanguageKey = 'PLG_ACTIONLOG_JOOMLA_' . $extensionType . '_UPDATED';
		} else {
			$messageLanguageKey = 'PLG_ACTIONLOG_JOOMLA_EXTENSION_UPDATED';
		}

		$message = array(
			'action'         => 'update',
			'type'           => 'PLG_ACTIONLOG_JOOMLA_TYPE_' . $extensionType,
			'id'             => $eid,
			'name'           => (string) $manifest->name,
			'extension_name' => (string) $manifest->name,
		);

		$this->addLog(array($message), $messageLanguageKey, $context);
	}

	/**
	 * On Saving extensions logging method
	 * Method is called when an extension is being saved
	 *
	 * @param   string   $context  The extension
	 * @param   Table    $table    DataBase Table object
	 * @param   boolean  $isNew    If the extension is new or not
	 *
	 * @return  void
	 *
	 */
	public function onExtensionAfterSave($context, $table, $isNew): void
	{
		if (!$this->params->get('extension_after_save')) {
			return;
		}

		$option = $this->app->input->getCmd('option');

		if ($table->get('module') != null) {
			$option = 'com_modules';
		}

		if (!$this->checkLoggable($option)) {
			return;
		}

		$params = ActionlogsHelper::getLogContentTypeParams($context);

		// Not found a valid content type, don't process further
		if ($params === null) {
			return;
		}

		list(, $contentType) = explode('.', $params->type_alias);

		if ($isNew) {
			$messageLanguageKey = $params->text_prefix . '_' . $params->type_title . '_ADDED';
			$defaultLanguageKey = 'PLG_SYSTEM_ACTIONLOGS_CONTENT_ADDED';
		} else {
			$messageLanguageKey = $params->text_prefix . '_' . $params->type_title . '_UPDATED';
			$defaultLanguageKey = 'PLG_SYSTEM_ACTIONLOGS_CONTENT_UPDATED';
		}

		// If the extension type doesn't have it own language key, use default language key
		if (!$this->app->getLanguage()->hasKey($messageLanguageKey)) {
			$messageLanguageKey = $defaultLanguageKey;
		}

		$message = array(
			'action'         => $isNew ? 'add' : 'update',
			'type'           => 'PLG_ACTIONLOG_JOOMLA_TYPE_' . $params->type_title,
			'id'             => $table->get($params->id_holder),
			'title'          => $table->get($params->title_holder),
			'extension_name' => $table->get($params->title_holder),
			'itemlink'       => ActionlogsHelper::getContentTypeLink($option, $contentType, $table->get($params->id_holder), $params->id_holder),
		);

		$this->addLog(array($message), $messageLanguageKey, $context);
	}

	/**
	 * On Deleting extensions logging method
	 * Method is called when an extension is being deleted
	 *
	 * @param   string  $context  The extension
	 * @param   Table   $table    DataBase Table object
	 *
	 * @return  void
	 *
	 */
	public function onExtensionAfterDelete($context, $table): void
	{
		if (!$this->params->get('extension_after_delete')) {
			return;
		}

		if (!$this->checkLoggable($this->app->input->get('option'))) {
			return;
		}

		$params = ActionlogsHelper::getLogContentTypeParams($context);

		// Not found a valid content type, don't process further
		if ($params === null) {
			return;
		}

		$messageLanguageKey = 'PLG_SYSTEM_ACTIONLOGS_CONTENT_DELETED';

		$message = array(
			'action' => 'delete',
			'type'   => 'PLG_ACTIONLOG_JOOMLA_TYPE_' . $params->type_title,
			'title'  => $table->get($params->title_holder),
		);

		$this->addLog(array($message), $messageLanguageKey, $context);
	}

	/**
	 * On saving user data logging method
	 *
	 * Method is called after user data is stored in the database.
	 * This method logs who created/edited any user's data
	 *
	 * @param   array    $user     Holds the new user data.
	 * @param   boolean  $isnew    True if a new user is stored.
	 * @param   boolean  $success  True if user was successfully stored in the database.
	 * @param   string   $msg      Message.
	 *
	 * @return  void
	 *
	 */
	public function onUserAfterSave($user, $isnew, $success, $msg): void
	{
		if (!$this->params->get('user_after_save')) {
			return;
		}

		$context = $this->app->input->get('option');
		$task    = $this->app->input->post->get('task');

		if (!$this->checkLoggable($context)) {
			return;
		}

		$jUser = Factory::getApplication()->getIdentity();

		if (!$jUser->id) {
			$messageLanguageKey = 'PLG_ACTIONLOG_JOOMLA_USER_REGISTERED';
			$action             = 'register';

			// Reset request
			if ($task === 'reset.request') {
				$messageLanguageKey = 'PLG_ACTIONLOG_JOOMLA_USER_RESET_REQUEST';
				$action             = 'resetrequest';
			}

			// Reset complete
			if ($task === 'reset.complete') {
				$messageLanguageKey = 'PLG_ACTIONLOG_JOOMLA_USER_RESET_COMPLETE';
				$action             = 'resetcomplete';
			}

			// Registration Activation
			if ($task === 'registration.activate') {
				$messageLanguageKey = 'PLG_ACTIONLOG_JOOMLA_USER_REGISTRATION_ACTIVATE';
				$action             = 'activaterequest';
			}
		} elseif ($isnew) {
			$messageLanguageKey = 'PLG_SYSTEM_ACTIONLOGS_CONTENT_ADDED';
			$action             = 'add';
		} else {
			$messageLanguageKey = 'PLG_SYSTEM_ACTIONLOGS_CONTENT_UPDATED';
			$action             = 'update';
		}

		$userId   = $jUser->id ?: $user['id'];
		$username = $jUser->username ?: $user['username'];

		$message = array(
			'action'      => $action,
			'type'        => 'PLG_ACTIONLOG_JOOMLA_TYPE_USER',
			'id'          => $user['id'],
			'title'       => $user['name'],
			'itemlink'    => 'index.php?option=com_users&task=user.edit&id=' . $user['id'],
			'userid'      => $userId,
			'username'    => $username,
			'accountlink' => 'index.php?option=com_users&task=user.edit&id=' . $userId,
		);

		$this->addLog(array($message), $messageLanguageKey, $context, $userId);
	}

	/**
	 * On deleting user data logging method
	 *
	 * Method is called after user data is deleted from the database
	 *
	 * @param   array    $user     Holds the user data
	 * @param   boolean  $success  True if user was successfully stored in the database
	 * @param   string   $msg      Message
	 *
	 * @return  void
	 *
	 */
	public function onUserAfterDelete($user, $success, $msg): void
	{
		if (!$this->params->get('user_after_delete')) {
			return;
		}

		$context = $this->app->input->get('option');

		if (!$this->checkLoggable($context)) {
			return;
		}

		$messageLanguageKey = 'PLG_SYSTEM_ACTIONLOGS_CONTENT_DELETED';

		$message = array(
			'action'      => 'delete',
			'type'        => 'PLG_ACTIONLOG_JOOMLA_TYPE_USER',
			'id'          => $user['id'],
			'title'       => $user['name'],
		);

		$this->addLog(array($message), $messageLanguageKey, $context);
	}

	/**
	 * On after save user group data logging method
	 *
	 * Method is called after user group is stored into the database
	 *
	 * @param   string   $context  The context
	 * @param   Table    $table    DataBase Table object
	 * @param   boolean  $isNew    Is new or not
	 *
	 * @return  void
	 *
	 */
	public function onUserAfterSaveGroup($context, $table, $isNew): void
	{
		if (!$this->params->get('user_after_save_group')) {
			return;
		}

		// Override context (com_users.group) with the component context (com_users) to pass the checkLoggable
		$context = $this->app->input->get('option');

		if (!$this->checkLoggable($context)) {
			return;
		}

		if ($isNew) {
			$messageLanguageKey = 'PLG_SYSTEM_ACTIONLOGS_CONTENT_ADDED';
			$action             = 'add';
		} else {
			$messageLanguageKey = 'PLG_SYSTEM_ACTIONLOGS_CONTENT_UPDATED';
			$action             = 'update';
		}

		$message = array(
			'action'      => $action,
			'type'        => 'PLG_ACTIONLOG_JOOMLA_TYPE_USER_GROUP',
			'id'          => $table->id,
			'title'       => $table->title,
			'itemlink'    => 'index.php?option=com_users&task=group.edit&id=' . $table->id,
		);

		$this->addLog(array($message), $messageLanguageKey, $context);
	}

	/**
	 * On deleting user group data logging method
	 *
	 * Method is called after user group is deleted from the database
	 *
	 * @param   array    $group    Holds the group data
	 * @param   boolean  $success  True if user was successfully stored in the database
	 * @param   string   $msg      Message
	 *
	 * @return  void
	 *
	 */
	public function onUserAfterDeleteGroup($group, $success, $msg): void
	{
		if (!$this->params->get('user_after_delete_group')) {
			return;
		}

		$context = $this->app->input->get('option');

		if (!$this->checkLoggable($context)) {
			return;
		}

		$messageLanguageKey = 'PLG_SYSTEM_ACTIONLOGS_CONTENT_DELETED';

		$message = array(
			'action'      => 'delete',
			'type'        => 'PLG_ACTIONLOG_JOOMLA_TYPE_USER_GROUP',
			'id'          => $group['id'],
			'title'       => $group['title'],
		);

		$this->addLog(array($message), $messageLanguageKey, $context);
	}

	/**
	 * Method to log user login success action
	 *
	 * @param   array  $options  Array holding options (user, responseType)
	 *
	 * @return  void
	 *
	 */
	public function onUserAfterLogin($options)
	{
		if (!$this->params->get('user_after_login')) {
			return;
		}

		if ($options['action'] === 'core.login.api') {
			return;
		}

		$context = 'com_users';

		if (!$this->checkLoggable($context)) {
			return;
		}

		$loggedInUser       = $options['user'];
		$messageLanguageKey = 'PLG_ACTIONLOG_JOOMLA_USER_LOGGED_IN';

		$message = array(
			'action'      => 'login',
			'userid'      => $loggedInUser->id,
			'username'    => $loggedInUser->username,
			'accountlink' => 'index.php?option=com_users&task=user.edit&id=' . $loggedInUser->id,
			'app'         => 'PLG_ACTIONLOG_JOOMLA_APPLICATION_' . $this->app->getName(),
		);

		$this->addLog(array($message), $messageLanguageKey, $context, $loggedInUser->id);
	}

	/**
	 * Method to log user login failed action
	 *
	 * @param   array  $response  Array of response data.
	 *
	 * @return  void
	 *
	 */
	public function onUserLoginFailure($response)
	{
		if (!$this->params->get('user_login_failure')) {
			return;
		}

		$context = 'com_users';

		if (!$this->checkLoggable($context)) {
			return;
		}

		// Get the user id for the given username
		$query = $this->db->getQuery(true)
			->select($this->db->quoteName(array('id', 'username')))
			->from($this->db->quoteName('#__users'))
			->where($this->db->quoteName('username') . ' = ' . $this->db->quote($response['username']));
		$this->db->setQuery($query);

		try {
			$loggedInUser = $this->db->loadObject();
		} catch (\Joomla\Database\Exception\ExecutionFailureException $e) {
			return;
		}

		// Not a valid user, return
		if (!isset($loggedInUser->id)) {
			return;
		}

		$messageLanguageKey = 'PLG_ACTIONLOG_JOOMLA_USER_LOGIN_FAILED';

		$message = array(
			'action'      => 'login',
			'id'          => $loggedInUser->id,
			'userid'      => $loggedInUser->id,
			'username'    => $loggedInUser->username,
			'accountlink' => 'index.php?option=com_users&task=user.edit&id=' . $loggedInUser->id,
			'app'         => 'PLG_ACTIONLOG_JOOMLA_APPLICATION_' . $this->app->getName(),
		);

		$this->addLog(array($message), $messageLanguageKey, $context, $loggedInUser->id);
	}

	/**
	 * Method to log user's logout action
	 *
	 * @param   array  $user     Holds the user data
	 * @param   array  $options  Array holding options (remember, autoregister, group)
	 *
	 * @return  void
	 *
	 */
	public function onUserLogout($user, $options = array())
	{
		if (!$this->params->get('user_logout')) {
			return;
		}

		$context = 'com_users';

		if (!$this->checkLoggable($context)) {
			return;
		}

		$loggedOutUser = User::getInstance($user['id']);

		if ($loggedOutUser->block) {
			return;
		}

		$messageLanguageKey = 'PLG_ACTIONLOG_JOOMLA_USER_LOGGED_OUT';

		$message = array(
			'action'      => 'logout',
			'id'          => $loggedOutUser->id,
			'userid'      => $loggedOutUser->id,
			'username'    => $loggedOutUser->username,
			'accountlink' => 'index.php?option=com_users&task=user.edit&id=' . $loggedOutUser->id,
			'app'         => 'PLG_ACTIONLOG_JOOMLA_APPLICATION_' . $this->app->getName(),
		);

		$this->addLog(array($message), $messageLanguageKey, $context);
	}

	/**
	 * On after Remind username request
	 *
	 * Method is called after user request to remind their username.
	 *
	 * @param   stdClass  $user  Holds the user data.
	 *
	 * @return  void
	 *
	 */
	public function onUserAfterRemind(stdClass $user)
	{
		if (!$this->params->get('user_after_remind')) {
			return;
		}

		$context = $this->app->input->get('option');

		if (!$this->checkLoggable($context)) {
			return;
		}

		$message = array(
			'action'      => 'remind',
			'type'        => 'PLG_ACTIONLOG_JOOMLA_TYPE_USER',
			'id'          => $user->id,
			'title'       => $user->name,
			'itemlink'    => 'index.php?option=com_users&task=user.edit&id=' . $user->id,
			'userid'      => $user->id,
			'username'    => $user->name,
			'accountlink' => 'index.php?option=com_users&task=user.edit&id=' . $user->id,
		);

		$this->addLog(array($message), 'PLG_ACTIONLOG_JOOMLA_USER_REMIND', $context, $user->id);
	}

	/**
	 * On after Check-in request
	 *
	 * Method is called after user request to check-in items.
	 *
	 * @param   array  $table  Holds the table name.
	 *
	 * @return  void
	 *
	 */
	public function onAfterCheckin($table)
	{
		if (!$this->params->get('after_checkin')) {
			return;
		}

		$context = 'com_checkin';
		$user    = Factory::getApplication()->getIdentity();

		if (!$this->checkLoggable($context)) {
			return;
		}

		$message = array(
			'action'      => 'checkin',
			'type'        => 'PLG_ACTIONLOG_JOOMLA_TYPE_USER',
			'id'          => $user->id,
			'title'       => $user->username,
			'itemlink'    => 'index.php?option=com_users&task=user.edit&id=' . $user->id,
			'userid'      => $user->id,
			'username'    => $user->username,
			'accountlink' => 'index.php?option=com_users&task=user.edit&id=' . $user->id,
			'table'       => str_replace($this->db->getPrefix(), '#__', $table),
		);

		$this->addLog(array($message), 'PLG_ACTIONLOG_JOOMLA_USER_CHECKIN', $context, $user->id);
	}

	/**
	 * On after log action purge
	 *
	 * Method is called after user request to clean action log items.
	 *
	 * @param   array  $group  Holds the group name.
	 *
	 * @return  void
	 *
	 */
	public function onAfterLogPurge($group = '')
	{
		if (!$this->params->get('after_log_purge')) {
			return;
		}

		$context = $this->app->input->get('option');
		$user    = Factory::getApplication()->getIdentity();
		$message = array(
			'action'      => 'actionlogs',
			'type'        => 'PLG_ACTIONLOG_JOOMLA_TYPE_USER',
			'id'          => $user->id,
			'title'       => $user->username,
			'itemlink'    => 'index.php?option=com_users&task=user.edit&id=' . $user->id,
			'userid'      => $user->id,
			'username'    => $user->username,
			'accountlink' => 'index.php?option=com_users&task=user.edit&id=' . $user->id,
		);
		$this->addLog(array($message), 'PLG_ACTIONLOG_JOOMLA_USER_LOG', $context, $user->id);
	}

	/**
	 * On after log export
	 *
	 * Method is called after user request to export action log items.
	 *
	 * @param   array  $group  Holds the group name.
	 *
	 * @return  void
	 *
	 */
	public function onAfterLogExport($group = '')
	{
		if (!$this->params->get('after_log_export')) {
			return;
		}

		$context = $this->app->input->get('option');
		$user    = Factory::getApplication()->getIdentity();
		$message = array(
			'action'      => 'actionlogs',
			'type'        => 'PLG_ACTIONLOG_JOOMLA_TYPE_USER',
			'id'          => $user->id,
			'title'       => $user->username,
			'itemlink'    => 'index.php?option=com_users&task=user.edit&id=' . $user->id,
			'userid'      => $user->id,
			'username'    => $user->username,
			'accountlink' => 'index.php?option=com_users&task=user.edit&id=' . $user->id,
		);
		$this->addLog(array($message), 'PLG_ACTIONLOG_JOOMLA_USER_LOGEXPORT', $context, $user->id);
	}

	/**
	 * On after Cache purge
	 *
	 * Method is called after user request to clean cached items.
	 *
	 * @param   string  $group  Holds the group name.
	 *
	 * @return  void
	 *
	 */
	public function onAfterPurge($group = 'all')
	{
		if (!$this->params->get('after_purge')) {
			return;
		}

		$context = $this->app->input->get('option');
		$user    = Factory::getApplication()->getIdentity();

		if (!$this->checkLoggable($context)) {
			return;
		}

		$message = array(
			'action'      => 'cache',
			'type'        => 'PLG_ACTIONLOG_JOOMLA_TYPE_USER',
			'id'          => $user->id,
			'title'       => $user->username,
			'itemlink'    => 'index.php?option=com_users&task=user.edit&id=' . $user->id,
			'userid'      => $user->id,
			'username'    => $user->username,
			'accountlink' => 'index.php?option=com_users&task=user.edit&id=' . $user->id,
			'group'       => $group,
		);
		$this->addLog(array($message), 'PLG_ACTIONLOG_JOOMLA_USER_CACHE', $context, $user->id);
	}

	/**
	 * On after Api dispatched
	 *
	 * Method is called after user perform an API request better on onAfterDispatch.
	 *
	 * @return  void
	 *
	 */
	public function onAfterDispatch()
	{
		if (!$this->params->get('after_dispatch')) {
			return;
		}

		if (!$this->app->isClient('api')) {
			return;
		}

		if ($this->loggableApi === 0) {
			return;
		}

		$verb = $this->app->input->getMethod();

		if (!in_array($verb, $this->loggableVerbs)) {
			return;
		}

		$context = $this->app->input->get('option');

		if (!$this->checkLoggable($context)) {
			return;
		}

		$user = $this->app->getIdentity();
		$message = array(
			'action'      => 'API',
			'verb'        => $verb,
			'username'    => $user->username,
			'accountlink' => 'index.php?option=com_users&task=user.edit&id=' . $user->id,
			'url'         => htmlspecialchars(urldecode($this->app->get('uri.route')), ENT_QUOTES, 'UTF-8'),
		);
		$this->addLog(array($message), 'PLG_ACTIONLOG_JOOMLA_API', $context, $user->id);
	}

	/**
	 * On after CMS Update
	 *
	 * Method is called after user update the CMS.
	 *
	 * @param   string  $oldVersion  The Joomla version before the update
	 *
	 * @return  void
	 *
	 */
	public function onJoomlaAfterUpdate($oldVersion = null)
	{
		if (!$this->params->get('joomla_after_update')) {
			return;
		}

		$context = $this->app->input->get('option');
		$user    = Factory::getApplication()->getIdentity();

		if (empty($oldVersion)) {
			$oldVersion = \Joomla\CMS\Language\Text::_('JLIB_UNKNOWN');
		}

		$message = array(
			'action'      => 'joomlaupdate',
			'type'        => 'PLG_ACTIONLOG_JOOMLA_TYPE_USER',
			'id'          => $user->id,
			'title'       => $user->username,
			'itemlink'    => 'index.php?option=com_users&task=user.edit&id=' . $user->id,
			'userid'      => $user->id,
			'username'    => $user->username,
			'accountlink' => 'index.php?option=com_users&task=user.edit&id=' . $user->id,
			'version'     => JVERSION,
			'oldversion'  => $oldVersion,
		);
		$this->addLog(array($message), 'PLG_ACTIONLOG_JOOMLA_USER_UPDATE', $context, $user->id);
	}
}
